'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
'  Poker Slots - Slot Machine Simulation Program
'  Steven Edwards
'  Version 1.00
'  20 September 1999
'-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

'  compiler switches :
'  I know this is redundant, call me a control freak
$APPTYPE  GUI
'  I need this to protect me from my typing mistakes
$TYPECHECK ON
'  lets pull in the include file
$INCLUDE "RAPIDQ.INC"

'  graphical resources :
'  I used .ICO files because I had them
$RESOURCE icoJOKER as "JOKER.ICO"
$RESOURCE icoHEART as "HEART.ICO"
$RESOURCE icoCLUB as "CLUB.ICO"
$RESOURCE icoDIAMOND as "DIAMOND.ICO"
$RESOURCE icoSPADE as "SPADE.ICO"
$RESOURCE icoLEMON as "LEMON.ICO"

'  program subroutines declarations :
'  with the typecheck ON we must declare subroutines and/or
'  functions that reside after code that makes calls to them
declare sub btnQuarterClick
declare sub btnSpinClick
declare sub Spinslots
declare sub CalcWinnings
declare function FormatMoney(money as single) as String

'  program constants
const StartMoney = 10.00
const MaxImageCount = 5
const Joker = 0
const Heart = 1
const Club = 2
const Diamond = 3
const Spade = 4
const Lemon = 5
const ImageTop = 128
const SpinTime = 125

'  global variables
dim SlotValue(4) as integer
dim SlotTimer(4) as integer
dim PlayerMoney as single
dim CurrentBet as single
dim Payoff as single
dim LastPayoff as single
dim SpinCounter as integer

'  define large red font used to display monies won/lost
create fDigital as qfont
    color = clRed
    size = 25
end create

'  define program gui
create frmMain as qform
    height = 350
    width = 288
    center
    borderstyle = bsSingle
    caption = "Poker Slots"
    icohandle = icoJoker
    '  the following "rules" are display labels only
    '  they are there strictly as a guide to what makes
    '  a winning "hand"
    create lblRule1 as qlabel
        top = 20
        left = 1
        width = 130
        alignment = taRightJustify
        caption = "Pair Wins 1x BET"
    end create
    create lblRule2 as qlabel
        top = 40
        left = 1
        width = 130
        alignment = taRightJustify
        caption = "Triple Wins 2x BET"
    end create
    create lblRule3 as qlabel
        top = 60
        left = 1
        width = 130
        alignment = taRightJustify
        caption = "2 Pair Wins 3x BET"
    end create
    create lblRule4 as qlabel
        top = 80
        left = 1
        width = 130
        alignment = taRightJustify
        caption = "4 of a Kind Wins 4x BET"
    end create
    create lblRule5 as qlabel
        top = 20
        left = 144
        width = 130
        alignment = taLeftJustify
        caption = "Lemons do NOT Count"
    end create
    create lblRule6 as qlabel
        top = 40
        left = 144
        width = 130
        alignment = taLeftJustify
        caption = "toward any win"
    end create
    create lblRule7 as qlabel
        top = 60
        left = 144
        width = 130
        alignment = taLeftJustify
        caption = "Jokers only count when"
    end create
    create lblRule8 as qlabel
        top = 80
        left = 144
        width = 130
        alignment = taLeftJustify
        caption = "4 Jokers Win 10x BET"
    end create
    '  the following images are the slot machine "wheels"
    '  that spin,  all will begin on a joker picture
    create imgSlot1 as qimage
        icohandle = icoJoker
        autosize = True
        top = ImageTop
        left = 32
    end create
    create imgSlot2 as qimage
        icohandle = icoJoker
        autosize = True
        top = ImageTop
        left = 96
    end create
    create imgSlot3 as qimage
        icohandle = icoJoker
        autosize = True
        top = ImageTop
        left = 160
    end create
    create imgSlot4 as qimage
        icohandle = icoJoker
        autosize = True
        top = ImageTop
        left = 224
    end create
    '  here is the button the player will click to begin the
    '  slot wheels spinning
    create btnSpin as Qbutton
        top = 200
        left = 200
        caption = "&Spin"
        onclick = btnSpinClick
        '  this button is not enabled until a "coin" has been
        '  dropped into the slot machine
        enabled = false
    end create
    '  here is the button that will simulate a player dropping
    '  a quarter into the slot machine
    create btnQuarter as qbutton
        top = 200
        left = 10
        caption = "Insert &Quarter"
        onclick = btnQuarterClick
    end create
    '  the following labels relate to the players money
    '  this is a descriptive label letting the player know
    '  what the next label is totaling
    create lblPlayerPocket as qlabel
        top = 250
        left = 10
        caption = "Player's Pocket"
    end create
    '  this label will be updated throughout the game as the
    '  player wins/loses money
    create lblPlayerMoney as qlabel
        top = 270
        left = 30
    end create
    '  "digital" readout of current bet / coins dropped into
    '  the slot machine
    create lblCurrentBet as qlabel
        top = 190
        left = 90
        width = 100
        height = 100
        alignment = taRightJustify
        font = fDigital
        caption = formatmoney(0)
    end create
    '  descriptive label identifies the current bet readout
    create lblCurrentBettxt as qlabel
        top = 230
        left = 115
        caption = "Current Bet"
    end create
    '  another "digital" readout showing what the last spin
    '  paid out to the player
    create lblLastPayoff as qlabel
        top = 250
        left = 90
        width = 100
        height = 100
        alignment = taRightJustify
        font = fDigital
        caption = formatmoney(Payoff)
    end create
    '   descriptive text identifies the last payoff readout
    create lblPayofftxt as qlabel
        top = 290
        left = 115
        caption = "Last Payoff"
    end create
end create

'  define timer for slot wheels to spin
create timSlotSpinner as qtimer
    enabled = false
    interval = SpinTime
    ontimer = SpinSlots
end create

'  function takes a SINGLE number and outputs a neatly
'  formatted string to maintain the dollars.cents (00.00)
'  for our readouts
function FormatMoney(money as single) as String
    dim moneyhold as integer
    dim stringhold as string
    dim dollars as string
    dim cents as string
    moneyhold = int(money)
    dollars = ltrim$(rtrim$(str$(moneyhold)))
    moneyhold = int((money-moneyhold) * 100)
    cents = left$(ltrim$(rtrim$(str$(moneyhold))) + "0000", 2)
    stringhold = dollars + "." + cents
    FormatMoney = stringhold
end function


'  subroutine to perform the action of the player dropping
'  a quarter into the machine
sub btnQuarterClick
    if PlayerMoney > 0 then
        PlayerMoney = PlayerMoney - .25
        CurrentBet = CurrentBet + .25
        lblCurrentBet.caption = formatmoney(CurrentBet)
        lblPlayerMoney.caption = formatmoney(PlayerMoney)
        btnSpin.enabled = true
    end if
end sub

'  subroutine to perform the action of the player initiating
'  the slot "wheels" spinning
sub btnSpinClick
    dim Holdtime as integer
    dim counter1 as integer
    LastPayOff = 0
    '  do not allow player to click any buttons until the spin
    '  is over
    btnQuarter.enabled = false
    btnSpin.enabled = false
    '  the following setion will let at least 10 wheel clicks
    '  roll before stopping each wheel one at a time
    Holdtime = 10
    for counter1 = 1 to 4
        HoldTime = HoldTime + int(rnd*5)
        SlotTimer(counter1) = HoldTime
    next counter1
    '  start the wheels turning
    SpinCounter = 0
    timSlotSpinner.interval = spinTime
    timSlotSpinner.enabled = true
end sub

'  the following subroutine is initiated by the TIMER event
'  it keeps track of the slot images rolling on the wheels
'  and updates them accordingly
sub SpinSlots
    dim HoldValue as integer
    dim Counter2 as integer
    SpinCounter = SpinCounter + 1
    for counter2 = 1 to 4
        if (SlotTimer(Counter2) > Spincounter) then
            SlotValue(Counter2) = SlotValue(Counter2) + 1
            if SlotValue(Counter2) > MaxImageCount then
                SlotValue(Counter2) = 0
            end if
            if Counter2 = 1 then
                select case SlotValue(Counter2)
                    case Joker
                        imgSlot1.icohandle = icoJoker
                    case Heart
                        imgSlot1.icohandle = icoHeart
                    case Spade
                        imgSlot1.icohandle = icoSpade
                    case Diamond
                        imgSlot1.icohandle = icoDiamond
                    case Club
                        imgSlot1.icohandle = icoClub
                    case Lemon
                        imgSlot1.icohandle = icoLemon
                    case else
                        frmMain.caption = "OOOPS!!!"
                end select
            end if
            if Counter2 = 2 then
                select case SlotValue(Counter2)
                    case Joker
                        imgSlot2.icohandle = icoJoker
                    case Heart
                        imgSlot2.icohandle = icoHeart
                    case Spade
                        imgSlot2.icohandle = icoSpade
                    case Diamond
                        imgSlot2.icohandle = icoDiamond
                    case Club
                        imgSlot2.icohandle = icoClub
                    case Lemon
                        imgSlot2.icohandle = icoLemon
                    case else
                        frmMain.caption = "OOOPS!!!"
                end select
            end if
            if Counter2 = 3 then
                select case SlotValue(Counter2)
                    case Joker
                        imgSlot3.icohandle = icoJoker
                    case Heart
                        imgSlot3.icohandle = icoHeart
                    case Spade
                        imgSlot3.icohandle = icoSpade
                    case Diamond
                        imgSlot3.icohandle = icoDiamond
                    case Club
                        imgSlot3.icohandle = icoClub
                    case Lemon
                        imgSlot3.icohandle = icoLemon
                    case else
                        frmMain.caption = "OOOPS!!!"
                end select
            end if
            if Counter2 = 4 then
                select case SlotValue(Counter2)
                    case Joker
                        imgSlot4.icohandle = icoJoker
                    case Heart
                        imgSlot4.icohandle = icoHeart
                    case Spade
                        imgSlot4.icohandle = icoSpade
                    case Diamond
                        imgSlot4.icohandle = icoDiamond
                    case Club
                        imgSlot4.icohandle = icoClub
                    case Lemon
                        imgSlot4.icohandle = icoLemon
                    case else
                        frmMain.caption = "OOOPS!!!"
                end select
            end if
        else
            if (SlotTimer(4) = Spincounter) then
                timSlotSpinner.enabled = false
                CalcWinnings
                btnQuarter.enabled = true
            end if
        end if
    next counter2
    timslotSpinner.interval = SpinTime
end sub

'  subroutine to determine if the images on the slot wheels
'  constitute a winning combination
sub CalcWinnings
    dim Counter as integer
    dim SuitCounts(MaxImageCount) as integer
    dim Done as integer
    dim Holder as integer
    dim Suits as string
    '  transfer Slot values to the local suit array
    for counter = 1 to 4
        SuitCounts(counter) = SlotValue(Counter)
    next counter
    '  sort array in numerical order
    do
        Done = true
        '  loop through array
        for Counter = 1 to 3
            '  check current counter element against next element
            if SuitCounts(counter) > SuitCounts(counter + 1) then
                '  swap elements
                Holder = SuitCounts(Counter)
                SuitCounts(Counter) = SuitCounts(Counter + 1)
                SuitCounts(Counter + 1) = Holder
                '  we have had a swap - we are not done
                Done = False
            end if
        next Counter
    loop until done
    '  convert the sorted array into a string
    Suits = ""
    For Counter = 1 to 4
        Suits = Suits + LTrim$(RTrim$(Str$(SuitCounts(Counter))))
    next Counter
    '  check for winning patterns
    Done = False
    Payoff = 0
    select case Suits
        '  check for obviuos winners
        case "0000"
            '  all JOKERS - largest payoff
            Payoff = CurrentBet * 10
            Done = True
        case "1111", "2222", "3333", "4444"
            '  4 of a kind
            Payoff = CurrentBet * 4
            Done = True
        case "1122", "1133", "1144", "2233", "2244", "3344"
            '  2 pair
            Payoff = CurrentBet * 3
            Done = True
        case "0111", "1112", "1113", "1114", "1115", _
             "0222", "1222", "2223", "2224", "2225", _
             "0333", "1333", "2333", "3334", "3335", _
             "0444", "1444", "2444", "3444", "4445"
            '  3 of a kind
            Payoff = CurrentBet * 2
            Done = true
        case "0555", "1555", "2555", "3555", "4555", "5555"
            '  obvius LOSS - too many lemons
            Done = true
        case else
            '  various other patterns
    end select
    '  Check if we have already found a pattern
    If Done = False then
        '  we have not found a pattern yet
        '  see if we have a pair of any suit
        if instr(Suits, "11") > 0 then
            '  we have found a pair of hearts
            Payoff = CurrentBet
        end if
        if instr(Suits, "22") > 0 then
            '  we have found a pair of clubs
            Payoff = CurrentBet
        end if
        if instr(Suits, "33") > 0 then
            '  we have found a pair of diamonds
            Payoff = CurrentBet
        end if
        if instr(Suits, "44") > 0 then
            '  we have found a pair of spades
            Payoff = CurrentBet
        end if
    end if
    '  clear bet
    CurrentBet = 0
    lblCurrentBet.caption = formatmoney(CurrentBet)
    '  show machine payoff
    LastPayoff = LastPayoff + Payoff
    lblLastPayoff.caption = formatmoney(LastPayoff)
    '  update player money
    PlayerMoney = PlayerMoney + Payoff
    lblPlayerMoney.caption = formatmoney(PlayerMoney)
end sub

'  Main Program Code
'  seed random number generator
randomize
'  intialize starting values
PlayerMoney = startmoney
lblPlayermoney.caption = formatmoney(PlayerMoney)
SlotValue(1) = 0
SlotValue(2) = 0
SlotValue(3) = 0
SlotValue(4) = 0
'  show form to player
frmMain.showmodal

